# -*- coding: utf-8 -*-
from __future__ import division
from kivy.app import App
from kivy.uix.widget import Widget
from kivy.uix.label import Label
from kivy.graphics import Ellipse, Line, Color, Rectangle
from kivy.core.window import Window

from copy import deepcopy
import threading
import random
import math

from Agent import Agent
from Space import Space
from Skaler import Skaler
from Mover import Mover

import numpy as np

class DrawerWidget(Widget): #Darstellende Teil von Kivy

    def __init__(self,agents):  #Erzeugen der Menschen als Ellipsen(Kivy) und Wände als Rechtecke(Rectangle,Kivy)
        super(DrawerWidget,self).__init__()
        self.agents = agents
        self.time = 0
        with self.canvas:
            self.counterlabel= Label(pos = (0,30),text='[b][color=3333ff]Anzahl EVAs:   0[/color][/b]', markup = True)
            self.timerlabel = Label(pos = (0,-20),text='[b][color=3333ff]Zeit in sec:   0[/color][/b]', markup = True)
            skal,offset = skaler.skale(Window.size[0],Window.size[1])
            for agent in self.agents:
                x = agent.x*skal + offset[0]
                y = agent.y*skal + offset[1]
                d = agent.d*skal
                agent.Vis= Ellipse(pos=((x-d/2),(y-d/2)), size = (d,d))
            for boundery in space.bounderies:
                x1 = boundery.pos1[0]*skal + offset[0]
                y1 = boundery.pos1[1]*skal + offset[1]
                x2 = boundery.pos2[0]*skal + offset[0]
                y2 = boundery.pos2[1]*skal + offset[1]
                width = (x2-x1)
                height = (y2-y1)
                boundery.Line = Rectangle(pos=(x1,y1), size = (width,height))
    
    def onResize(self,parent,w,h):  #Skaliert mit der neuen Bildschirmgröße
        with self.canvas:
            skal,offset = skaler.skale(Window.size[0],Window.size[1])
            for agent in self.agents:
                d = agent.d*skal
                agent.Vis.pos = (agent.pos[0]*skal+offset[0]-d/2,agent.pos[1]*skal+offset[1]-d/2)
            for boundery in space.bounderies:
                x1 = boundery.pos1[0]*skal + offset[0]
                y1 = boundery.pos1[1]*skal + offset[1]
                x2 = boundery.pos2[0]*skal + offset[0]
                y2 = boundery.pos2[1]*skal + offset[1]
                width = (x2-x1)
                height = (y2-y1)
                boundery.Line.pos = (x1,y1)
                boundery.Line.size = (width,height)


    def moving(self, counter,t):    #bekommt die neue Position des Menschen, skaliert diese und zeichnet sie neu
        self.time += t
        with self.canvas:
            self.counterlabel.text = str('[b][color=3333ff] Anzahl Evas:     '+str(counter)+'[/color][/b]')
            self.timerlabel.text = str('[b][color=3333ff] Zeit in sec:     '+str(self.time)+'[/color][/b]')
            skal,offset = skaler.skale(Window.size[0],Window.size[1])
            for agent in self.agents:
                d = agent.d*skal
                agent.Vis.pos = (agent.pos[0]*skal+offset[0]-d/2,agent.pos[1]*skal+offset[1]-d/2)
            

class DrawerApp(App):   #Hauptprogramm von Kivy
    def build(self):
        self.parent = Widget()
        self.painter = DrawerWidget(agents) #Der eigentliche Teil der zeichnet
        self.parent.add_widget(self.painter)
        Window.bind(on_resize=self.painter.onResize)    #Damit der Raum immer an das Fenster angepasst wird
        return self.parent

def createAgents(width,height,main=None):
    d = 0.5
    w = space.wallWidth
    a = True
    while a:    # Überprüfng ob sich der neue Mensch in anderen Menschen oder Hindernissen befindet
        x=d/2+w+random.random()*(width-(d+2*w))
        y=d/2+w+random.random()*(height-(d+2*w))
        testpos = np.array([x,y])
        if (len(agents)!=0):
            for agent in agents:
                if math.sqrt((agent.pos[0]-x)**2+(agent.pos[1]-y)**2)>=(d+agent.d)/2:
                    a=False
                else:
                    a=True
                    break
        else: a = False
        h1 = np.array([d/2,d/2])
        for boun in space.bounderies:
            if np.all(np.logical_and(boun.pos1-h1 <= testpos, testpos <= boun.pos2+h1)): a = True
    agents.append(Agent(x,y,d))


#Hier wird das eigentliche Programm gestartet
if __name__=='__main__':
    newModel = int(raw_input("Neues Modell?  "))
    if newModel == 0:
        model = int(raw_input("Modell:  "))
    else: model = "new"
    agents = []
    space = Space(model)    #erzeugung Space
    n=int(raw_input('Anzahl der Menschen: '))
    for i in range(n):
        createAgents(space.width,space.height)
    skaler = Skaler(space.width,space.height)
    app = DrawerApp()
    m = Mover(agents,space) # Dieser läuft im Hintergrund und berechnet die neuen Positionen der Menschen
    thread1 = threading.Thread(target=m.force, args=(app,))
    thread1.start()
    app.run()
