#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

from __future__ import division
import numpy as np
import skfmm
from scipy import interpolate
from Boundery import Boundery as Boundary

#Dieser Programmteil wurde uns gegeben, das Verständnis ist nicht notwendig
def ist_in_Wand(pos,wand):
	p1 = wand.pos1
        p2 = wand.pos2
        laenge = p2[0] - p1[0]
        hoehe = p2[1] - p1[1]
        center = 0.5*(p1+p2)
	v1 = np.array([1,0])
        v2 = np.array([0,1])
	return (abs(np.dot(pos-center,v1))<laenge/2) and \
	       (abs(np.dot(pos-center,v2))<hoehe/2)
	       
	    

def Laufwegkarte(Wand_Liste,Anfangs_Liste,xmin,xmax,ymin,ymax,n=200,speed=None):
	'''Mit Hilfe der fast marching method (skfmm) wird die Eikonalgleichung
	gelöst. Die Wellenausbreitung beginnt bei den Rechtecken, die zu den Wänden in
	Anfangs_Liste gehören, wobei die Rechtecke, die zu den Wänden in der Wand_Liste
	gehören, undurchdringlich sind.
	Die Funktion liefert (X,Y,T) zurück, wobei X, Y Arrays der x,y-Werte sind und
	T ein nxn-Array des (zeitlichen) Abstands zur Anfangs_Liste.	
	Optional kann noch ein nxn-Array übergeben speed übergeben werden, das die
	jeweilige lokale Ausbreitungsgeschwindigkeit enthält.'''
	
	X, Y = np.meshgrid(np.linspace(xmin-5,xmax+5,n), np.linspace(ymin-5,ymax+5,n))
	phi = 1.*np.ones_like(X)
	if speed is None:
		speed=np.ones_like(X)
	mask=speed>1
	for i in range(n):
		for j in range(n):
			x=X[i,j]
			y=Y[i,j]
			for Wand in Wand_Liste:
				if ist_in_Wand((x,y),Wand):
					mask[i,j]=True
			for Wand in Anfangs_Liste:
				if ist_in_Wand((x,y),Wand):
					phi[i,j]=-1.
					mask[i,j]=False
	phi  = np.ma.MaskedArray(phi, mask)
	T= skfmm.travel_time(phi, speed, dx=[(xmax-xmin)*1./n,(ymax-ymin)*1./n])
	print T
	return X,Y,T,phi
	
	
def make_direction_function(Wand_Liste,Anfangs_Liste,xmin,xmax,ymin,ymax,n=200,speed=None):
	'''Diese Funktion gibt eine FUNKTION zurück, die die Richtung aufs Ziel im
	Sinne des schnellsten Wegs berechnet.
	Etwa:
	
	g=make_direction_function(....)
	x=10.
	y=23.
	richtung=g([x,y])
	Die Argumente entsprechen der Funktion Laufwegkarte(...)
	'''
	
	X,Y,T,phi=Laufwegkarte(Wand_Liste,Anfangs_Liste,xmin,xmax,ymin,ymax,n=n,speed=None)
	Tgrad=np.gradient(T,(xmax-xmin)*1.0/n,(ymax-ymin)*1.0/n)
	try:
		Tgrad[0][Tgrad[0].mask]=0
		Tgrad[1][Tgrad[1].mask]=0
		Tgrad[0].mask=False
		Tgrad[1].mask=False
	except:
		pass
	points=np.concatenate((X.reshape((n*n,1)),Y.reshape((n*n,1))),axis=1)
	g1=interpolate.NearestNDInterpolator(points, -Tgrad[1].flatten())
	g2=interpolate.NearestNDInterpolator(points, -Tgrad[0].flatten())
	#Diese lambda Funktion ermöglicht die übergabe einer Position und gibt den Richtungvektor dieser Position zurück
	return lambda v: np.array([g1(v[0],v[1]),g2(v[0],v[1])])
	
	
	
if __name__=='__main__':
	import pylab as pl
	Wand1 = Boundary( pos1=np.array([0,-20]), pos2= np.array([200,20]))
	Wand2= Boundary( pos1=np.array([0,-20]), pos2= np.array([20,200]))
	Wand_Liste=[Wand1,Wand2]

        laenge = 400
        breite = 400
        
	Ausgang =  Boundary( pos1=np.array([-40,-100]), pos2= np.array([-30,-90]))
	X,Y,T,phi=Laufwegkarte(Wand_Liste,[Ausgang],-laenge/1.8, laenge/1.8, -breite/1.8, breite/1.8,n=50)
	Tgrad=np.gradient(T)
	
	
	
	pl.contour(X, Y, phi.mask, [0], linewidths=(3), colors='red')
	pl.contour(X, Y, phi,[0], linewidths=(3), colors='black')
	pl.contour(X,Y,T,15)
	
	M=4 # jeden M. Punkt für die Auswertung des Vektorfelds
	pl.quiver(X[::M,::M],Y[::M,::M],-10*Tgrad[1][::M,::M],-10*Tgrad[0][::M,::M])
	pl.show()

	df=make_direction_function(Wand_Liste,[Ausgang],-laenge/1.8, laenge/1.8, -breite/1.8, breite/1.8,n=50)
	n=50
	vectorfield=np.zeros((2,(n-1)//M+1,(n-1)//M+1))
	for i in range((n-1)//M+1):
		for j in range((n-1)//M+1):
			vectorfield[:,i,j]=df([X[i*M,j*M],Y[i*M,j*M]])
	pl.quiver(X[::M,::M],Y[::M,::M],10*vectorfield[0],10*vectorfield[1],15)
	pl.show()
#X, Y = np.meshgrid(np.linspace(-1,1,200), np.linspace(-1,1,200))
#phi = -1*np.ones_like(X)

##phi[X>-0.5] = 1
##phi[np.logical_and(np.abs(Y)<0.25, X>-0.75)] = 1
#phi[np.logical_or(np.abs(X)>0.5,np.abs(Y)>0.5)]=1


#pl.contour(X, Y, phi,[0], linewidths=(3), colors='black')
#pl.title('Boundary location: the zero contour of phi')
#pl.savefig('2d_phi.png')
#pl.show()

#d = skfmm.distance(phi, dx=1e-2)
#pl.title('Distance from the boundary')
#pl.contour(X, Y, phi,[0], linewidths=(3), colors='black')
#pl.contour(X, Y, d, 15)
#pl.colorbar()
#pl.savefig('2d_phi_distance.png')
#pl.show()

#speed = np.ones_like(X)
#speed[Y>0] = 1.5
#t = skfmm.travel_time(phi, speed, dx=1e-2)

#pl.title('Travel time from the boundary')
#pl.contour(X, Y, phi,[0], linewidths=(3), colors='black')
#pl.contour(X, Y, t, 15)
#pl.colorbar()
#pl.savefig('2d_phi_travel_time.png')
#pl.show()

#mask = np.logical_and(abs(X)<0.1, abs(Y)<0.3)
#phi  = np.ma.MaskedArray(phi, mask)
#t    = skfmm.travel_time(phi, speed, dx=1e-2)
#pl.title('Travel time from the boundary with an obstacle')
#pl.contour(X, Y, phi, [0], linewidths=(3), colors='black')
#pl.contour(X, Y, phi.mask, [0], linewidths=(3), colors='red')
#pl.contour(X, Y, t, 15)
#pl.colorbar()
#pl.savefig('2d_phi_travel_time_mask.png')
#pl.show()
